// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// Fichier contenant le header des routines de gestion EEPROM
//
// Ce fichier contient la structure des donnees memorisees en EEPROM
//=======================================
// History:
// 24/10/04    V1.0
//       Version originale
//=======================================

#ifndef TIPLL_EEPROM_H
#define TIPLL_EERPOM_H

/* include system */
#include <avr/eeprom.h>

/* include applicatif */
#include "TIPLL_FreqMgt.h"

/* definition de la structure des donnees */

#define EEP_FREQ_DEF_VAL   FREQ_QUICK_2   /* frequence par defaut */
#define EEP_FREQ1_VAL      FREQ_QUICK_1
#define EEP_FREQ2_VAL      FREQ_QUICK_2
#define EEP_FREQ3_VAL      FREQ_QUICK_3
#define EEP_PROG_VAL          1  /* programmation frequence a l'init par defaut */

typedef struct  { 
  uint16_t  FreqDefault;                        /* Frequence par defaut en 10KHz */
  uint16_t  FreqQuick[FREQ_NB_QUICK_FREQ];      /* Frequence rapide  (en 10KHz) */
  uint16_t  Prog_FreqDefault;                   /* ce flag permet de shunter la prog de la frequence initiale si Disable (=0)*/ 

  /* fin des donnees utiles de l'application */
  uint16_t  Validite_EEPROM;                    /* controle validite EEPROM checksum desdonnees utiles de l'EEPROM */   
} T_TIPLL_EEPROM_data ;




/* fonctions applicatives de gestion EEPROM */
void EEP_Init(void);                       /* initialisation pointeur, controle validite et init par defaut en cas de donnes non valides */
void EEP_SetDefault(void);                 /* initialisation par defaut de l'EEPROM */
char EEP_Check(void);                      /* controle la validite de l'EEPROM , retourne 0 si OK, -1 si KO */
void EEP_set_FreqDefault(uint16_t FreqDef);     /* memorisation Frequence par defaut */
void EEP_set_FreqQuick(unsigned char Indice,uint16_t FreqDef);  /* memorisation Frequence rapide */
void EEP_set_FlagProg(uint16_t Flag);           /* memorisation flag de programmation initiale */

uint16_t EEP_get_FreqDefault(void);             /* retourne la Frequence par defaut */
uint16_t EEP_get_FreqQuick(unsigned char Indice);        /* retourne la  Frequence rapide */
uint16_t EEP_get_FlagProg(void);                /* retroune le flag de programmation initiale */

uint16_t EEP_update_valid(void);                /* calcule la checksum de validite, la memorise et la retourne */

#endif
