// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// Fichier contenant le header des routines de gestion des Entrees/sorties logiques
//
//=======================================
// History:
// 29/10/04    V1.0
//       Version originale
// 12/04/06    V1.1
//       Adaptation pour la version TI PFM
// 03/11/06    V1.2
//       Correction des tempos IO_FILTRE_MS et IO_FILTRE_REPEAT pour Q 16MHz
//=======================================

#ifndef TIPLL_IOMGT_H
#define TIPLL_IOMGT_H

/* include system */


/* definition des entrees */
/* modif PFM UP : D3-> D6; DOWN : D3->D7;  QUICK : D4 inchange ; ajout STORE : D5 */
#define IO_BP_UP             6   /* Bouton Poussoir touche UP PORTD.6 (actif etat bas)  */
#define IO_BP_DOWN           7   /* Bouton Poussoir touche DOWN PORTD.7 (actif etat bas)  */
#define IO_BP_QUICK          4   /* Bouton Poussoir touche quick PORTD.4 (actif etat bas)  */
#define IO_BP_STORE          5   /* Bouton Poussoir touche STORE (ESC PFM) PORTD.5 (actif etat bas)  */
#define IO_LOCK_DETECT       0   /* Etat de sortie PLL PORTB.0 (actif etat bas )*/

/* defintions des sorties */
#define IO_LED_LOCK          1  /* LED lock PLL sur PORTB.1 actif etat haut */
/* modif PFM IO_LED_BL_LOCK non utilise */
/* #define IO_LED_BL_LOCK       7 */    /* LED Back Light LOCK PORTD.7 LOCK si etat bas */ 
#define IO_LCD_RS            0  /* PORT C  RS LCD si 0 registre ctrl/1=data */
#define IO_LCD_E             1  /* PORT C selection LCD sur front desendant */
#define IO_LCD_RW            2  /* PORT C Read/Write LCD */
/* GL 13/10/06 correction LCF READY D3 ->B4*/
/* modif PFM LCD_READY C4 --> B4 !!!!!! attention port B !!!!!! */
#define IO_LCD_READY         4  /* PORT B.4 (= DB7 LCD 1 si ready) */

/* definition des constantes de filtrage anti rebond */
#define IO_FILTRE_MS        4  /* filtre anti rebond  65.536mS =  4 x boucles totale de 16.384 msec (1 pas =64uS) 256 pas d'horloge */
#define IO_FILTRE_REPEAT    8  /* timer pour touche auto repeat 131 ms ( soit ~7 autorepeat/sec)*/


/* definition des types */
typedef enum {
	IO_NO_BP_USED=0,  /* aucune touche active */
	IO_UP,            /* touche Up active */
	IO_DOWN,          /* touche Down acive */
	IO_QUICK,          /* touche Quick active */
	IO_STORE			/* modif PFM touche STORE active */
} T_E_IO_CLAVIER;     /* Enum des sorties possibles du clavier */

typedef enum {
	IO_PLL_UNLOCK=0,   /* PLL non verrouillee */
	IO_PLL_LOCK        /* PLL verrouillee */
} T_E_IO_PLL;        

/* fonctions applicatives de gestion IO */

void IO_init(void);                         /* init port E/S et valeur initiale + init timer 1  */

T_E_IO_CLAVIER  IO_read_keyboard(void);     /* lecture du clavier , si une touche est activee attente tempo anti rebond */

unsigned char IO_Keyboard_AutoRepeat(void); /* retourne 0 si l'autorepeat n'est pas actif */

T_E_IO_PLL  IO_read_LockDetect(void);     /* lecture lock detect PLL */

void IO_set_LEDLock(T_E_IO_PLL LEDStatus);   /* LED allumee si IO_PLL_LOCK */

T_E_IO_PLL IO_refresh_LEDLock(void);       /* lit l'etat du lock detect , positionne la LED en consequence et retourne cette valeur */

void IO_wait_ms(unsigned int WaitTime);    /* attente tempo de  16.384mS*WaitTime en utilisant le timer 0 */

#endif

