// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// derived from Application note AVR306
// adaptation and modification from GL 
//=======================================
// History:
// 20/10/04    V1.0
//     first version from original application note
//     replace all UBRR0H, UBRR0L, etc by UBRRH,UBRRL etc ...
// 21/12/04    V1.1
//     Utilisation de SIGNAL a la place de INTERRUPT sinon boucle infinie de traitemetnt IT
//     Ajout des fonctions de debug en transmission polling (permet d'avoir la trace au moment de l'execution, sinon perte du timing r? d'execution)
// 05/12/06    V1.2
//     mise en commentaire des  dernieres traces de debug
//=======================================

/* Includes */
#include "TIPLL_USART2.h"
#include <avr/interrupt.h>

#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>   

/* Static Variables */
static unsigned char USART_RxBuf[USART_RX_BUFFER_SIZE];
static volatile unsigned char USART_RxHead;
static volatile unsigned char USART_RxTail;
static unsigned char USART_TxBuf[USART_TX_BUFFER_SIZE];

static volatile unsigned char USART_TxHead;
static volatile unsigned char USART_TxTail;

static unsigned char USART_NbCmd;   /* compteur du nombre de commandes dans le buffer de reception */
static unsigned char USART_TmpConvertString[20];  /* conversion en ascii */

void USART_decNbCmd(void)
{
        cli();  /* masque IT pour section critique */
        USART_NbCmd--;
        sei();  /* autorisation IT fin de section critique */
}

unsigned char USART_getNbCmd(void)
{
	return(USART_NbCmd);
}


/* Initialize USART */
void USART0_Init( unsigned int baudrate )
{
	unsigned char x;
	

	/* Set the baud rate */
	UBRRH = (unsigned char) (baudrate>>8);                  
	UBRRL = (unsigned char) baudrate;
	
	/* Enable UART receiver and transmitter */
	UCSRB = ( ( 1 << RXCIE ) | ( 1 << RXEN ) | ( 1 << TXEN ) ); 
	
	/* Set frame format: 8 data 2stop */
	/* URSEL positionne a 1 pour selectionner UCSRC (@ commune avec UBRRH) */
	UCSRC = (1<<URSEL)|(1<<USBS)|(1<<UCSZ1)|(1<<UCSZ0);  
	
	/* Flush receive buffer */
	x = 0; 			    

	USART_RxTail = x;
	USART_RxHead = x;
	USART_TxTail = x;
	USART_TxHead = x;
    USART_NbCmd = x;

}

/* *************************************** */
/* ***       Interrupt handlers        *** */
/* *************************************** */

/* interrupt handler pour reception */
/* -------------------------------- */

/*  ATTENTION utilisation de SIGNAL et non INTERRUPT */
/* SIGNAL n'autorise pas les nouvelles IT avant la fin du traitement*/
/* alors que INTERRUPT commence par un "sei()" , une nouvelle IT est generee sans pouvoir l'acquiter */
SIGNAL(SIG_UART_RECV)
{
	unsigned char data;
	unsigned char tmphead;

	/* Read the received data */
	data = UDR;                 
	/* Calculate buffer index */
	tmphead = ( USART_RxHead + 1 ) & USART_RX_BUFFER_MASK;
	USART_RxHead = tmphead;      /* Store new index */

	if ( tmphead == USART_RxTail )
	{
		/* ERROR! Receive buffer overflow */
		/* pas de risque d'interbloquage, car buffer circulaire */
	    USART0_Transmit_Poll( '!' ); /* envoi caractere erreur */
	}
	else
	{
	    USART0_Transmit_Poll( data ); /* Echo the received character */
    }
	
	USART_RxBuf[tmphead] = data; /* Store received data in buffer */
	
	if (data == '\r')
	{
	   /* si "return" alors incrementation nombre de commandes */
	   USART_NbCmd++;	
	}
}

/* interrupt handler pour l'emission */
/* --------------------------------- */

/* utilisation de SIG_UART_DATA correspondant a TDRE au lieu de SIG_UART_TRANS */
/* ATTENTION utilisation de SIGNAL au lieu de INTERRUPT !!!!!!!! */
SIGNAL(SIG_UART_DATA)
{
	unsigned char tmptail;

	/* Check if all data is transmitted */
	if ( USART_TxHead != USART_TxTail )
	{
		/* Calculate buffer index */
		tmptail = ( USART_TxTail + 1 ) & USART_TX_BUFFER_MASK;
		USART_TxTail = tmptail;      /* Store new index */
	
		UDR = USART_TxBuf[tmptail];  /* Start transmition */
	}
	else
	{
		UCSRB &= ~(1<<UDRIE);         /* Disable UDRE interrupt */
	}
}

/* fonctions bas niveau d'ecriture et de lecture */
/* --------------------------------------------- */

unsigned char USART0_Receive( void )
{
	unsigned char tmptail;
	
	while ( USART_RxHead == USART_RxTail )  /* Wait for incomming data */
		;
	tmptail = ( USART_RxTail + 1 ) & USART_RX_BUFFER_MASK;/* Calculate buffer index */
	
	USART_RxTail = tmptail;                /* Store new index */
	
	return USART_RxBuf[tmptail];           /* Return data */
}

void USART0_Transmit( unsigned char data )
{
    
   /* Dans cette version l'envoie de caracteres est uniquement disponible en polling*/
   /* Les Interruptions ne sont utilisees qu'en reception */    

   /* pour le debug envoi uniquement en polling */
   USART0_Transmit_Poll(data);

}
void USART0_Transmit_Poll( unsigned char data )
{

	/* polling  wait end sending */
	while ( !(UCSRA & (1<<UDRE)));
    UDR= data; 
}

/* fonction retournant 0 (FALSE) si le buffer de reception est vide */
unsigned char DataInReceiveBuffer( void )
{
	return ( USART_RxHead != USART_RxTail ); 
}

/* lit un caractere depuis le buffer de commande et le retourne */
unsigned char USART_ReadCmdChar(void)
{
     unsigned char TmpChar;
     
     TmpChar=USART0_Receive();
     
     if (TmpChar == '\r')
     {
           USART_decNbCmd(); /* decrementation compteur commande */     	
     }
     return(TmpChar);
}

/* lit un parametre Unsigned Int et le retourne apres conversion */
unsigned int  USART_ReadCmdParamUi(void)
{
	char TmpConvString[10];
	
	/* lecture de la chaine*/
	USART_ReadCmdParamStr(TmpConvString);
	return (atoi(TmpConvString));
}


/* lit une chaine delimitee par ' ' ou '\r' de car et la retourne  */
/* prevoir un param supplementaire pour limiter la taille de la chaine (risque de debordement en cas d'erreur de syntaxe) */

void     USART_ReadCmdParamStr(char * ParamReadOut)
{
     unsigned char TmpChar;
     
	 // USART_PrintfStr(PSTR("USART_ReadCmdParamStr")); 
	 
     TmpChar=USART_ReadCmdChar();
     /* Suppression des premiers espaces separateurs */
     while (TmpChar == ' ')
     {
         TmpChar=USART_ReadCmdChar();
     }
     while ( (TmpChar != ' ') && (TmpChar != '\r') )
     {
     	*ParamReadOut=TmpChar;
     	ParamReadOut++;
        TmpChar=USART_ReadCmdChar();
     }
     *ParamReadOut=0;   /* delimiteur de fin de chaine*/
}






/* printf polling vers la liaison serie depuis une chaine de caracteres en Flash */
void USART_PrintfStr(PGM_P StringChar )
{

    /* printf vers la liaison serie */
	while (pgm_read_byte(StringChar) !='\0')
	{
	    USART0_Transmit_Poll(pgm_read_byte(StringChar) ); /* emission du caractere */
	    StringChar++; /* incrementation pointeur */
	};
}

/* printf polling vers la liaison serie depuis une chaine de caracteres en RAM */
void USART_PrintfStrRAM( unsigned char  * StringChar )
{

    /* printf vers la liaison serie */
	while (*StringChar !='\0')
	{
	    USART0_Transmit_Poll( *StringChar ); /* emission du caractere */
	    StringChar++; /* incrementation pointeur */
	};
}

/* printf polling vers la liaison serie d'un caractere */
void USART_Printfc( unsigned char  Caractere )
{
    /* printf vers la liaison serie d'un caractere */
	USART0_Transmit_Poll( Caractere ); /* emission du caractere */
}

/* printf standard vers la liaison serie d'une chaine de caracteres en RAM */
void USART_PrintfStr_IT( unsigned char  * StringChar )
{
    /* printf vers la liaison serie avec IT */
	while (*StringChar !=0)
	{
	    USART0_Transmit( *StringChar ); /* emission du caractere */
	    StringChar++; /* incrementation pointeur */
	};
}

/* printf standard vers la liaison serie d'un caractere */
void USART_Printfc_IT( unsigned char  Caractere )
{
    /* printf vers la liaison serie d'un caractere avec IT */
	USART0_Transmit( Caractere ); /* emission du caractere */
}

/* printf standard formate d'un integer (16b) passe en argument*/
/* format : 1 : Hexa 0x */
/*          0 : decimal */
void USART_PrintfUi_IT( unsigned int  Data, unsigned char Format )
{
    /* printf d'un unsigend int vers la liaison serie avec IT selon format 0: 10, 1:hexa */

	
    switch (Format)
    {
      case 1:
       {
       	  USART_PrintfStr(PSTR("0x"));
       	  utoa(Data,USART_TmpConvertString,16);
       }
       break;
       case 0:
       default:
       {
       	  utoa(Data,USART_TmpConvertString,10);
       }
    }
    USART_PrintfStrRAM(USART_TmpConvertString);
}

